<?php
namespace Neklo\Scheduler\Controller\Adminhtml\Schedule;

use Neklo\Scheduler\Controller\Adminhtml;
use Neklo\Scheduler\Api\Data\SchedulerInterface;

class Edit extends Adminhtml\Action
{
    /**
     * Redirect url
     */
    const REDIRECT_URL = '*/index/index';
    const MODEL_REGISTRY = 'edit_schedule';

    protected function _initAction($resultPage)
    {
        parent::_initAction($resultPage);
        $resultPage->setActiveMenu('Neklo_Scheduler::view');
    }

    public function execute()
    {
        $ret = $this->setupModel();
        if ($ret !== false) {
            return $ret;
        }

        $resultPage = $this->resultPageFactory->create();
        $this->_initAction($resultPage);
        $resultPage->addBreadcrumb(
            __('Edit Schedule'),
            __('Edit Schedule'));
        $resultPage->getConfig()->getTitle()->prepend(__('Cron Scheduler'));
        return $resultPage;
    }

    protected function setupModel()
    {
        $id = $this->getRequest()->getParam(SchedulerInterface::SCHEDULE_ID);
        $model = $this->_objectManager->create('Magento\Cron\Model\Schedule');

        if ($id) {
            $model->load($id);
            if (empty($model->getData())) {
                $this->messageManager
                    ->addError(__('This scheduled job no longer exists.'));
                return $this->_redirect(static::REDIRECT_URL);
            } else {
                // 3. Set entered data if was error when we do save
                $data = $this->_objectManager
                    ->get('Magento\Backend\Model\Session')
                    ->getFormData(true);
                if (!empty($data)) {
                    $model->setData($data);
                }
                // 4. Register model to use later in blocks
                $this->coreRegistry->register(static::MODEL_REGISTRY, $model);
            }
        } else {
            return $this->_redirect(static::REDIRECT_URL);
        }
        return false;
    }

    /**
     * Check if admin has permissions to visit related pages
     *
     * @return bool
     */
    protected function _isAllowed()
    {
        return $this->_authorization
            ->isAllowed('Neklo_Scheduler::edit');
    }
}
