<?php
namespace Neklo\Scheduler\Helper;

class Job extends \Magento\Framework\App\Helper\AbstractHelper
    implements \Magento\Cron\Model\ConfigInterface
{

    /**
     *
     * @var array
     */
    protected $jobs;
    /**
     * Cron config data
     *
     * @var \Magento\Cron\Model\Config\Data
     */
    protected $_configData;

    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Cron\Model\Config\Data $configData
    ) {
        parent::__construct($context);
        $this->_configData = $configData;
    }

    public function getJobCodes($group = null)
    {
        $codes = [];
        $jobs = $this->getJobs($group);
        if (is_null($group)) {
            foreach ($jobs as $groupJobs) {
                $codes = array_merge($codes, array_keys($groupJobs));
            }
        } else {
            $codes = array_keys($jobs);
        }

        // Sort
        sort($codes);

        return $codes;
    }

    /**
     *
     * @param string $group Cron job group name. Returns all by default
     */
    public function getJobs($group = null)
    {
        if (is_null($this->jobs)) {
            $this->jobs = $this->_configData->getJobs();;
        }
        if (is_null($group)) {
            return $this->jobs;
        } else {
            if (isset($this->jobs[$group])) {
                return $this->jobs[$group];
            } else {
                throw new \Exception(__("Job group '%1' does not exist", $group));
            }
        }
    }

    public function getJobNameOptions($group = null)
    {
        $names = $this->getJobCodes($group);
        $ret = [];
        foreach ($names as $name)
        {
            $ret[] = ['value' => $name, 'label' => $name];
        }
        return $ret;
    }

    /**
     * Validates cron schedule expression
     * @param string $schedule
     */
    public function validateSchedule($schedule)
    {
        $result = preg_match(
//            "/(\*|[0-5]?[0-9]|\*\/[0-9]+)\s+"
//            ."(\*|1?[0-9]|2[0-3]|\*\/[0-9]+)\s+"
//            ."(\*|[1-2]?[0-9]|3[0-1]|\*\/[0-9]+)\s+"
//            ."(\*|[0-9]|1[0-2]|\*\/[0-9]+|jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec)\s+"
//            ."(\*\/[0-9]+|\*|[0-7]|sun|mon|tue|wed|thu|fri|sat)\s*"
//            ."(\*\/[0-9]+|\*|[0-9]+)?/i"
            '/^(\*|((\*\/)?[1-5]‌​?[0-9])) '.
            '(\*|((\*\/)?[1-5]?[0‌​-9])) '.
            '(\*|((\*\/)?(1?[0-9]‌​|2[0-3]))) '.
            '(\*|((\*\/)?([1-9]|[‌​12][0-9]|3[0-1]))) '.
            '(\*|((\*\/)?([1-9]|1‌​[0-2]))) '.
            '(\*|((\*\/)?[0-6]))$‌​/i',
            $schedule);
        return $result === 1;
    }

    public function isEnabled($name)
    {
        #TODO
        return true;
    }
}
